@@ -126,7 +126,7 @@ clfftStatus clfftEnqueueTransform(
 	{
 		case CLFFT_1D:
 		{
-			if (fftPlan->length[0] <= Large1DThreshold)
+			if ( Is1DPossible(fftPlan->length[0], Large1DThreshold) )
 				break;
 
 			if( ( fftPlan->inputLayout == CLFFT_REAL ) && ( fftPlan->planTZ != 0) )
@@ -225,38 +225,52 @@ clfftStatus clfftEnqueueTransform(
 					_T("clfftEnqueueTransform large1D col pass failed"));
 
 
-				// another column FFT output, INPLACE
-				OPENCL_V( clfftEnqueueTransform( fftPlan->planY, CLFFT_FORWARD, numQueuesAndEvents, commQueues, 1, &colOutEvents,
-					&copyInEvents, &(fftPlan->intBufferRC), &(fftPlan->intBufferRC), localIntBuffer ),
-					_T("clfftEnqueueTransform large1D second column failed"));
-				clReleaseEvent(colOutEvents);
-
 				cl_mem *out_local;
 				out_local = (fftPlan->placeness==CLFFT_INPLACE) ? clInputBuffers : clOutputBuffers;
 
+
+				// another column FFT output, INPLACE
+				OPENCL_V(clfftEnqueueTransform(fftPlan->planY, CLFFT_FORWARD, numQueuesAndEvents, commQueues, 1, &colOutEvents,
+					&copyInEvents, &(fftPlan->intBufferRC), &(fftPlan->intBufferRC), localIntBuffer),
+					_T("clfftEnqueueTransform large1D second column failed"));
+				clReleaseEvent(colOutEvents);
+
 				// copy from full complex to hermitian
-				OPENCL_V( clfftEnqueueTransform( fftPlan->planRCcopy, CLFFT_FORWARD, numQueuesAndEvents, commQueues, 1, &copyInEvents,
-					outEvents, &(fftPlan->intBufferRC), out_local, localIntBuffer ),
+				OPENCL_V(clfftEnqueueTransform(fftPlan->planRCcopy, CLFFT_FORWARD, numQueuesAndEvents, commQueues, 1, &copyInEvents,
+					outEvents, &(fftPlan->intBufferRC), out_local, localIntBuffer),
 					_T("clfftEnqueueTransform large1D RC copy failed"));
 				clReleaseEvent(copyInEvents);
 
+
 			}
 			else if( fftPlan->outputLayout == CLFFT_REAL )
 			{
 				cl_event colOutEvents = NULL;
 				cl_event copyOutEvents = NULL;
 
-				// copy from hermitian to full complex
-				OPENCL_V( clfftEnqueueTransform( fftPlan->planRCcopy, CLFFT_BACKWARD, numQueuesAndEvents, commQueues, numWaitEvents,
-					waitEvents, &copyOutEvents, clInputBuffers, &(fftPlan->intBufferRC), localIntBuffer ),
-					_T("clfftEnqueueTransform large1D RC copy failed"));
+				if (fftPlan->planRCcopy)
+				{
+					// copy from hermitian to full complex
+					OPENCL_V(clfftEnqueueTransform(fftPlan->planRCcopy, CLFFT_BACKWARD, numQueuesAndEvents, commQueues, numWaitEvents,
+						waitEvents, &copyOutEvents, clInputBuffers, &(fftPlan->intBufferRC), localIntBuffer),
+						_T("clfftEnqueueTransform large1D RC copy failed"));
 
-				// First pass
-				// column with twiddle first, INPLACE,
-				OPENCL_V( clfftEnqueueTransform( fftPlan->planX, CLFFT_BACKWARD, numQueuesAndEvents, commQueues, 1,
-					&copyOutEvents, &colOutEvents, &(fftPlan->intBufferRC), &(fftPlan->intBufferRC), localIntBuffer),
-					_T("clfftEnqueueTransform large1D col pass failed"));
-				clReleaseEvent(copyOutEvents);
+					// First pass
+					// column with twiddle first, INPLACE,
+					OPENCL_V(clfftEnqueueTransform(fftPlan->planX, CLFFT_BACKWARD, numQueuesAndEvents, commQueues, 1,
+						&copyOutEvents, &colOutEvents, &(fftPlan->intBufferRC), &(fftPlan->intBufferRC), localIntBuffer),
+						_T("clfftEnqueueTransform large1D col pass failed"));
+					clReleaseEvent(copyOutEvents);
+				}
+				else
+				{
+					// First pass
+					// column with twiddle first, INPLACE,
+					OPENCL_V(clfftEnqueueTransform(fftPlan->planX, CLFFT_BACKWARD, numQueuesAndEvents, commQueues, numWaitEvents,
+						waitEvents, &colOutEvents, clInputBuffers, &(fftPlan->intBufferRC), localIntBuffer),
+						_T("clfftEnqueueTransform large1D col pass failed"));
+					clReleaseEvent(copyOutEvents);
+				}
 
 				cl_mem *out_local;
 				out_local = (fftPlan->placeness==CLFFT_INPLACE) ? clInputBuffers : clOutputBuffers;
@@ -303,9 +317,18 @@ clfftStatus clfftEnqueueTransform(
 					//First transpose
 					// Input->tmp
 					cl_event transTXOutEvents = NULL;
-					OPENCL_V( clfftEnqueueTransform( fftPlan->planTX, dir, numQueuesAndEvents, commQueues, numWaitEvents,
-						waitEvents, &transTXOutEvents, clInputBuffers, &localIntBuffer, NULL ),
-						_T("clfftEnqueueTransform for large1D transTX failed"));
+					if(fftPlan->allOpsInplace)
+					{
+						OPENCL_V( clfftEnqueueTransform( fftPlan->planTX, dir, numQueuesAndEvents, commQueues, numWaitEvents,
+							waitEvents, &transTXOutEvents, clInputBuffers, NULL, NULL ),
+							_T("clfftEnqueueTransform for large1D transTX failed"));
+					}
+					else
+					{
+						OPENCL_V( clfftEnqueueTransform( fftPlan->planTX, dir, numQueuesAndEvents, commQueues, numWaitEvents,
+							waitEvents, &transTXOutEvents, clInputBuffers, &localIntBuffer, NULL ),
+							_T("clfftEnqueueTransform for large1D transTX failed"));
+					}
 
 					cl_mem *mybuffers;
 					if (fftPlan->placeness==CLFFT_INPLACE)
@@ -325,9 +348,18 @@ clfftStatus clfftEnqueueTransform(
 					//First Row
 					//tmp->output
 					cl_event rowXOutEvents = NULL;
-					OPENCL_V( clfftEnqueueTransform( fftPlan->planX, dir, numQueuesAndEvents, commQueues, 1,
-						&transTXOutEvents, &rowXOutEvents, &localIntBuffer, mybuffers, NULL ),
-						_T("clfftEnqueueTransform for large1D rowX failed"));
+					if(fftPlan->allOpsInplace)
+					{
+						OPENCL_V( clfftEnqueueTransform( fftPlan->planX, dir, numQueuesAndEvents, commQueues, 1,
+							&transTXOutEvents, &rowXOutEvents, clInputBuffers, NULL, NULL ),
+							_T("clfftEnqueueTransform for large1D rowX failed"));
+					}
+					else
+					{
+						OPENCL_V( clfftEnqueueTransform( fftPlan->planX, dir, numQueuesAndEvents, commQueues, 1,
+							&transTXOutEvents, &rowXOutEvents, &localIntBuffer, mybuffers, NULL ),
+							_T("clfftEnqueueTransform for large1D rowX failed"));
+					}
 					clReleaseEvent(transTXOutEvents);
 
 
@@ -343,9 +375,18 @@ clfftStatus clfftEnqueueTransform(
 					//Second Transpose
 					// output->tmp
 					cl_event transTYOutEvents = NULL;
-					OPENCL_V( clfftEnqueueTransform( fftPlan->planTY, dir, numQueuesAndEvents, commQueues, 1,
-						&rowXOutEvents, &transTYOutEvents, mybuffers, &localIntBuffer, NULL ),
-						_T("clfftEnqueueTransform for large1D transTY failed"));
+					if(fftPlan->allOpsInplace)
+					{
+						OPENCL_V( clfftEnqueueTransform( fftPlan->planTY, dir, numQueuesAndEvents, commQueues, 1,
+							&rowXOutEvents, &transTYOutEvents, clInputBuffers, NULL, NULL ),
+							_T("clfftEnqueueTransform for large1D transTY failed"));
+					}
+					else
+					{
+						OPENCL_V( clfftEnqueueTransform( fftPlan->planTY, dir, numQueuesAndEvents, commQueues, 1,
+							&rowXOutEvents, &transTYOutEvents, mybuffers, &localIntBuffer, NULL ),
+							_T("clfftEnqueueTransform for large1D transTY failed"));
+					}
 					clReleaseEvent(rowXOutEvents);
 
 
@@ -361,9 +402,18 @@ clfftStatus clfftEnqueueTransform(
 					//Second Row
 					//tmp->tmp, inplace
 					cl_event rowYOutEvents = NULL;
-					OPENCL_V( clfftEnqueueTransform( fftPlan->planY, dir, numQueuesAndEvents, commQueues, 1,
-						&transTYOutEvents, &rowYOutEvents, &localIntBuffer, NULL, NULL ),
-						_T("clfftEnqueueTransform for large1D rowY failed"));
+					if(fftPlan->allOpsInplace)
+					{
+						OPENCL_V( clfftEnqueueTransform( fftPlan->planY, dir, numQueuesAndEvents, commQueues, 1,
+							&transTYOutEvents, &rowYOutEvents, clInputBuffers, NULL, NULL ),
+							_T("clfftEnqueueTransform for large1D rowY failed"));
+					}
+					else
+					{
+						OPENCL_V( clfftEnqueueTransform( fftPlan->planY, dir, numQueuesAndEvents, commQueues, 1,
+							&transTYOutEvents, &rowYOutEvents, &localIntBuffer, NULL, NULL ),
+							_T("clfftEnqueueTransform for large1D rowY failed"));
+					}
 					clReleaseEvent(transTYOutEvents);
 
 #if defined(DEBUGGING)
@@ -377,9 +427,18 @@ clfftStatus clfftEnqueueTransform(
 
 					//Third Transpose
 					// tmp->output
-					OPENCL_V( clfftEnqueueTransform( fftPlan->planTZ, dir, numQueuesAndEvents, commQueues, 1,
-						&rowYOutEvents, outEvents, &localIntBuffer, mybuffers, NULL ),
-						_T("clfftEnqueueTransform for large1D transTZ failed"));
+					if(fftPlan->allOpsInplace)
+					{
+						OPENCL_V( clfftEnqueueTransform( fftPlan->planTZ, dir, numQueuesAndEvents, commQueues, 1,
+							&rowYOutEvents, outEvents, clInputBuffers, NULL, NULL ),
+							_T("clfftEnqueueTransform for large1D transTZ failed"));
+					}
+					else
+					{
+						OPENCL_V( clfftEnqueueTransform( fftPlan->planTZ, dir, numQueuesAndEvents, commQueues, 1,
+							&rowYOutEvents, outEvents, &localIntBuffer, mybuffers, NULL ),
+							_T("clfftEnqueueTransform for large1D transTZ failed"));
+					}
 					clReleaseEvent(rowYOutEvents);
 
 				}
@@ -452,9 +511,9 @@ clfftStatus clfftEnqueueTransform(
 						{
 							cl_event colOutEvents = NULL;
 							// First pass
-							// column with twiddle first, OUTOFPLACE, + transpose
+							// column with twiddle first, OUTOFPLACE
 							OPENCL_V( clfftEnqueueTransform( fftPlan->planX, dir, numQueuesAndEvents, commQueues, numWaitEvents,
-								waitEvents, &colOutEvents, clInputBuffers, &localIntBuffer, localIntBuffer),
+								waitEvents, &colOutEvents, clInputBuffers, &localIntBuffer, NULL),
 								_T("clfftEnqueueTransform large1D col pass failed"));
 
 #if defined(DEBUGGING)
@@ -489,11 +548,11 @@ clfftStatus clfftEnqueueTransform(
 							}
 							else
 							{
-								//another column FFT output, OUTOFPLACE
+								//another column FFT output, OUTOFPLACE + transpose
 								if (fftPlan->placeness == CLFFT_INPLACE)
 								{
 									OPENCL_V( clfftEnqueueTransform( fftPlan->planY, dir, numQueuesAndEvents, commQueues, 1, &colOutEvents,
-										outEvents, &localIntBuffer, clInputBuffers, localIntBuffer ),
+										outEvents, &localIntBuffer, clInputBuffers, NULL ),
 										_T("clfftEnqueueTransform large1D second column failed"));
 
 #if defined(DEBUGGING)
@@ -513,7 +572,7 @@ clfftStatus clfftEnqueueTransform(
 									_T("Reading the result buffer failed") );
 #endif
 									OPENCL_V( clfftEnqueueTransform( fftPlan->planY, dir, numQueuesAndEvents, commQueues, 1, &colOutEvents,
-										outEvents, &localIntBuffer, clOutputBuffers, localIntBuffer ),
+										outEvents, &localIntBuffer, clOutputBuffers, NULL ),
 										_T("clfftEnqueueTransform large1D second column failed"));
 
 #if defined(DEBUGGING)
@@ -556,7 +615,7 @@ clfftStatus clfftEnqueueTransform(
 
 			if( fftRepo.pStatTimer )
 			{
-				fftRepo.pStatTimer->AddSample( plHandle, fftPlan, NULL, 0, NULL, std::vector< size_t >( ) );
+				fftRepo.pStatTimer->AddSample( plHandle, fftPlan, NULL, 0, NULL, std::vector< size_t >( ), std::vector< size_t >() );
 			}
 
 			return	CLFFT_SUCCESS;
@@ -940,7 +999,7 @@ clfftStatus clfftEnqueueTransform(
 					{
 						//deal with row first
 						OPENCL_V( clfftEnqueueTransform( fftPlan->planX, dir, numQueuesAndEvents, commQueues, numWaitEvents,
-							waitEvents, &rowOutEvents, clInputBuffers, &localIntBuffer, localIntBuffer ),
+							waitEvents, &rowOutEvents, clInputBuffers, &localIntBuffer, NULL ),
 							_T("clfftEnqueueTransform for row failed"));
 
 
@@ -948,14 +1007,14 @@ clfftStatus clfftEnqueueTransform(
 						{
 							//deal with column
 							OPENCL_V( clfftEnqueueTransform( fftPlan->planY, dir, numQueuesAndEvents, commQueues, 1, &rowOutEvents,
-								outEvents, &localIntBuffer, clInputBuffers, localIntBuffer ),
+								outEvents, &localIntBuffer, clInputBuffers, NULL ),
 								_T("clfftEnqueueTransform for column failed"));
 						}
 						else
 						{
 							//deal with column
 							OPENCL_V( clfftEnqueueTransform( fftPlan->planY, dir, numQueuesAndEvents, commQueues, 1, &rowOutEvents,
-								outEvents, &localIntBuffer, clOutputBuffers, localIntBuffer ),
+								outEvents, &localIntBuffer, clOutputBuffers, NULL ),
 								_T("clfftEnqueueTransform for column failed"));
 
 			#if defined(DEBUGGING)
@@ -974,7 +1033,7 @@ clfftStatus clfftEnqueueTransform(
 
 			if( fftRepo.pStatTimer )
 			{
-				fftRepo.pStatTimer->AddSample( plHandle, fftPlan, NULL, 0, NULL, std::vector< size_t >( ) );
+				fftRepo.pStatTimer->AddSample( plHandle, fftPlan, NULL, 0, NULL, std::vector< size_t >( ), std::vector< size_t >() );
 			}
 
 			return	CLFFT_SUCCESS;
@@ -1222,7 +1281,7 @@ clfftStatus clfftEnqueueTransform(
 
 			if( fftRepo.pStatTimer )
 			{
-				fftRepo.pStatTimer->AddSample( plHandle, fftPlan, NULL, 0, NULL, std::vector< size_t >( ) );
+				fftRepo.pStatTimer->AddSample( plHandle, fftPlan, NULL, 0, NULL, std::vector< size_t >( ), std::vector< size_t >() );
 			}
 
 			return	CLFFT_SUCCESS;
