@@ -131,7 +131,7 @@ private:
 
 	buffer<T> input;
 	buffer<T> output;
-
+	
 	size_t number_of_data_points;
 	T _forward_scale, _backward_scale;
 	cl_uint commandQueueFlags;
@@ -617,6 +617,96 @@ public:
 		input = other_buffer;
 	}
 
+	/*****************************************************/
+	void set_input_precallback(unsigned int localMemSize = 0) {
+		cl_int status = 0;
+		clfftPrecision precision;
+		clfftGetPlanPrecision( *plan_handle, &precision );
+
+		const char* precallbackstr;
+		
+		if (localMemSize > 0)
+		{
+			//Test for LDS in precallback function
+			precallbackstr = STRINGIFY(MULVAL_LDS);
+		}
+		else
+		{
+			if (input.is_interleaved() )
+			{
+				precallbackstr = (precision == CLFFT_SINGLE) ? STRINGIFY(MULVAL) : STRINGIFY(MULVAL_DP);
+			}
+			else if (input.is_planar())
+			{
+				precallbackstr = (precision == CLFFT_SINGLE) ? STRINGIFY(MULVAL_PLANAR) : STRINGIFY(MULVAL_PLANAR_DP);
+			}
+			else if (input.is_real())
+			{
+				precallbackstr = (precision == CLFFT_SINGLE) ? STRINGIFY(MULVAL_REAL) : STRINGIFY(MULVAL_REAL_DP);
+			}
+		}
+
+		//precallback user data
+		buffer<T> userdata( 	static_cast<size_t>(dimension),
+					input.lengths(),
+					input.strides(),
+					input.batch_size(),
+					input.distance(),
+					layout::real,
+					_placeness
+					);
+
+		userdata.set_all_to_random_data(lengths[0], 10);
+		
+		// make the new buffer
+		const size_t bufferSizeBytes = userdata.size_in_bytes( );
+
+		cl_mem userdataBuff = clCreateBuffer( context.get( ), CL_MEM_READ_ONLY | CL_MEM_COPY_HOST_PTR, bufferSizeBytes, userdata.real_ptr(), &status);
+		OPENCL_V_THROW( status, "Creating Buffer ( ::clCreateBuffer() )" );
+
+		//Register the callback
+		OPENCL_V_THROW (clfftSetPlanCallback(*plan_handle, "mulval", precallbackstr, localMemSize, PRECALLBACK, &userdataBuff, 1), "clFFTSetPlanCallback failed");
+	}
+
+		/*****************************************************/
+	void set_input_precallback_userdatatype() {
+		cl_int status = 0;
+
+		char* precallbackstr = STRINGIFY(MULVAL_UDT);
+
+		size_t totalPts = input.total_number_of_points_including_data_and_intervening();
+
+		buffer<T> temp( 	static_cast<size_t>(dimension),
+					input.lengths(),
+					input.strides(),
+					input.batch_size(),
+					input.distance(),
+					layout::real,
+					_placeness
+					);
+
+		temp.set_all_to_random_data(lengths[0], 10);
+
+		std::vector<USER_DATA> userdata(totalPts);
+		size_t the_index;
+		for( size_t batch = 0; batch < input.batch_size(); batch++) 
+			for( size_t z = 0; z < input.length(dimz); z++) 
+				for( size_t y = 0; y < input.length(dimy); y++) 
+					for( size_t x = 0; x < input.length(dimx); x++) 
+					{
+						the_index = ( input.stride(dimx) * x + input.stride(dimy) * y + input.stride(dimz) * z + input.distance() * batch );
+
+						userdata[the_index].scalar1 = (float)temp.real(x, y, z, batch);
+						userdata[the_index].scalar2 = 1;
+					}
+
+		cl_mem userdataBuff = clCreateBuffer(context.get( ), CL_MEM_READ_ONLY | CL_MEM_COPY_HOST_PTR, sizeof(USER_DATA) * totalPts, (void*)&userdata[0], &status);
+		OPENCL_V_THROW( status, "Creating Buffer ( ::clCreateBuffer() )" );
+
+		//Register the callback
+		OPENCL_V_THROW (clfftSetPlanCallback(*plan_handle, "mulval", precallbackstr, 0, PRECALLBACK, &userdataBuff, 1), "clFFTSetPlanCallback failed");
+	}
+
 	/*****************************************************/
 	bool device_list_has_devices() {
 		return !device_id.empty();
